/*****************************************************************************
 * 'main' illustrates that the default copy constructor and default assignment 
 * operator just copy data members of class objects, not the contents of any 
 * dynamically allocated memory associated with the object. 
 *
 * 'Pair' objects store just a pointer to memory dynamically allocated upon
 * construction. The default copy constructor copies the value of the pointer,
 * leaving two objects pointing to the same memory.
 *
 * Likewise, the default assignment operator makes the left hand object's 
 * pointer equal to that of the right hand object. They both point at the same
 * memory, rather than having independent copies of the same doubles. 
 * Additionally, the memory originally allocated to the left hand object
 * remains allocated. If no other Pair refers to this memory, we have a memory
 * leak, memory that is permanently allocated, but of no use to the program.
 * **************************************************************************/
#include<iostream>
#include<iomanip>

using std::cin; using std::cout;
using std::endl;
class Pair
{
	double* p;
	public:
	Pair()
	{
		p=new double[2];
	}
	Pair( double x, double y)
	{
		p= new double[2];
		p[0]=x;
		p[1]=y;
	}
	void set_x(double z)
	{
		p[0]=z;
	}
	double get_x() const
	{
		return p[0];
	}
	void set_y(double z) const
	{
		p[1]=z;
	}
	double get_y()
	{
		return p[1];
	}
};

int main()
{
	Pair ones(1.0,1.1);
	cout<<"ones: "<<ones.get_x()<<", "<<ones.get_y()<<endl;
	Pair others;
	cout<<"others: "<<others.get_x()<<", "<<others.get_y()<<endl;
	others.set_x(3.3);
	others.set_y(4.5);
	cout<<"others: "<<others.get_x()<<", "<<others.get_y()<<endl;
	others=ones;// copies the 'p' value from 'ones' to 'others'
	cout<<"others: "<<others.get_x()<<", "<<others.get_y()<<endl;
	others.set_x(6.02);
	others.set_y(300000);
	cout<<"others: "<<others.get_x()<<", "<<others.get_y()<<endl;
	cout<<"ones: "<<ones.get_x()<<", "<<ones.get_y()<<endl;
/* Note that the assignment made the pointers in the two Pairs equal.
 * 'ones' and 'others' now point to the same memory. The memory 
 * allocated to 'others' remains allocated, but we no longer have a
 * pointer to it! This is a memory leak.
 * */
	Pair more=ones;// another pointer copy
	more.set_x(666);
	cout<<"more: "<<more.get_x()<<", "<<more.get_y()<<endl;
	cout<<"ones: "<<ones.get_x()<<", "<<ones.get_y()<<endl;
/* 'ones', 'others', and 'more' wind up pointing to the same memory. */	


	return 0;
}

	
